import { Plugin, PluginKey } from 'prosemirror-state';
import { DecorationSet, Decoration } from 'prosemirror-view';
import { toggleMark } from 'prosemirror-commands';
import { isMarkActive, isNodeActive, isListItem, removeLink, calculateBubblePos, isLinkActive, getSelectionMarks } from 'ngx-editor/helpers';
import { toggleList, toggleBlockType, toggleWrap } from 'ngx-editor/commands';

const DEFAULT_PLACEHOLDER = 'Type Here...';
const PLACEHOLDER_CLASSNAME = 'NgxEditor__Placeholder';
function placeholderPlugin(text = DEFAULT_PLACEHOLDER) {
    return new Plugin({
        key: new PluginKey('placeholder'),
        props: {
            decorations(state) {
                var _a;
                const doc = state.doc;
                if (doc.childCount === 1 && ((_a = doc === null || doc === void 0 ? void 0 : doc.firstChild) === null || _a === void 0 ? void 0 : _a.isTextblock) && doc.firstChild.content.size === 0) {
                    const placeHolderEl = document.createElement('span');
                    placeHolderEl.classList.add(PLACEHOLDER_CLASSNAME);
                    placeHolderEl.textContent = text;
                    return DecorationSet.create(doc, [Decoration.widget(1, placeHolderEl)]);
                }
                return DecorationSet.empty;
            }
        }
    });
}

/* tslint:disable */
var bold = `
  <path d="M15.6 10.79c.97-.67 1.65-1.77 1.65-2.79 0-2.26-1.75-4-4-4H7v14h7.04c2.09 0 3.71-1.7 3.71-3.79 0-1.52-.86-2.82-2.15-3.42zM10 6.5h3c.83 0 1.5.67 1.5 1.5s-.67 1.5-1.5 1.5h-3v-3zm3.5 9H10v-3h3.5c.83 0 1.5.67 1.5 1.5s-.67 1.5-1.5 1.5z" />
  <path d="M0 0h24v24H0z" fill="none" />
`;

var italic = `
  <path d="M0 0h24v24H0z" fill="none" />
  <path d="M10 4v3h2.21l-3.42 8H6v3h8v-3h-2.21l3.42-8H18V4z" />
`;

var code = `
<path d="M0 0h24v24H0V0z" fill="none"/>
<path d="M9.4 16.6L4.8 12l4.6-4.6L8 6l-6 6 6 6 1.4-1.4zm5.2 0l4.6-4.6-4.6-4.6L16 6l6 6-6 6-1.4-1.4z"/>
`;

var orderedList = `
<path d="M2 17h2v.5H3v1h1v.5H2v1h3v-4H2v1zm1-9h1V4H2v1h1v3zm-1 3h1.8L2 13.1v.9h3v-1H3.2L5 10.9V10H2v1zm5-6v2h14V5H7zm0 14h14v-2H7v2zm0-6h14v-2H7v2z"/>
<path d="M0 0h24v24H0z" fill="none"/>
`;

var bulletList = `
<path d="M0 0h24v24H0V0z" fill="none"/>
<path d="M4 10.5c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5zm0-6c-.83 0-1.5.67-1.5 1.5S3.17 7.5 4 7.5 5.5 6.83 5.5 6 4.83 4.5 4 4.5zm0 12c-.83 0-1.5.68-1.5 1.5s.68 1.5 1.5 1.5 1.5-.68 1.5-1.5-.67-1.5-1.5-1.5zM7 19h14v-2H7v2zm0-6h14v-2H7v2zm0-8v2h14V5H7z"/>
`;

var quote = `
<path d="M0 0h24v24H0z" fill="none"/><path d="M6 17h3l2-4V7H5v6h3zm8 0h3l2-4V7h-6v6h3z"/>
`;

var link = `
<path d="M0 0h24v24H0z" fill="none"/><path d="M3.9 12c0-1.71 1.39-3.1 3.1-3.1h4V7H7c-2.76 0-5 2.24-5 5s2.24 5 5 5h4v-1.9H7c-1.71 0-3.1-1.39-3.1-3.1zM8 13h8v-2H8v2zm9-6h-4v1.9h4c1.71 0 3.1 1.39 3.1 3.1s-1.39 3.1-3.1 3.1h-4V17h4c2.76 0 5-2.24 5-5s-2.24-5-5-5z"/>
`;

var unlink = `
<path d="M0 0h24v24H0V0z" fill="none"/>
<path d="M17 7h-4v1.9h4c1.71 0 3.1 1.39 3.1 3.1 0 1.43-.98 2.63-2.31 2.98l1.46 1.46C20.88 15.61 22 13.95 22 12c0-2.76-2.24-5-5-5zm-1 4h-2.19l2 2H16zM2 4.27l3.11 3.11C3.29 8.12 2 9.91 2 12c0 2.76 2.24 5 5 5h4v-1.9H7c-1.71 0-3.1-1.39-3.1-3.1 0-1.59 1.21-2.9 2.76-3.07L8.73 11H8v2h2.73L13 15.27V17h1.73l4.01 4L20 19.74 3.27 3 2 4.27z"/>
<path d="M0 24V0" fill="none"/>
`;

// Icons source: https://material.io/
const DEFAULT_ICON_HEIGHT = 20;
const DEFAULT_ICON_WIDTH = 20;
const icons = {
    bold,
    italic,
    code,
    ordered_list: orderedList,
    bullet_list: bulletList,
    quote,
    link,
    unlink
};
// Helper function to create menu icons
function getIconSvg(name, width = DEFAULT_ICON_WIDTH, height = DEFAULT_ICON_HEIGHT) {
    const path = icons[name] || '<path></path>';
    return `
  <svg
    xmlns="http://www.w3.org/2000/svg"
    viewBox="0 0 24 24"
    fill="black"
    height=${height}
    width=${width}
  >
    ${path}
  </svg>
  `;
}

const flatDeep = (arr, d = 1) => {
    return d > 0
        ? arr.reduce((acc, val) => acc.concat(Array.isArray(val) ? flatDeep(val, d - 1) : val), [])
        : arr.slice();
};
const ɵ0 = flatDeep;

class MenuItem {
    constructor(spec) {
        this.render = () => {
            const { spec, dom } = this;
            if (spec.classNames) {
                spec.classNames.forEach(className => {
                    dom.classList.add(className);
                });
            }
            if (spec.attrs) {
                Object.entries(spec.attrs).forEach(obj => {
                    dom.setAttribute(obj[0], obj[1]);
                });
            }
            if (spec.icon) {
                const icon = document.createElement('div');
                icon.className = spec.iconContainerClass;
                icon.innerHTML = spec.icon.default;
                this.iconContainer = icon;
                dom.appendChild(icon);
            }
            else if (spec.textContent) {
                dom.textContent = spec.textContent;
            }
            else {
                dom.textContent = '';
            }
        };
        this.update = (options) => {
            const { dom, spec } = this;
            dom.classList.toggle(spec.activeClass, options.active);
            dom.classList.toggle(spec.disabledClass, options.disabled);
        };
        this.toggleIcon = (toggle) => {
            const { iconContainer, spec } = this;
            if (!iconContainer) {
                return;
            }
            iconContainer.innerHTML = toggle ? spec.icon.alt : spec.icon.default;
        };
        this.dom = document.createElement('div');
        this.spec = spec;
        this.render();
    }
}

const MENU_ITEM_CLASSNAME = 'NgxEditor__MenuItem';
const MENU_ITEM_ICON_CLASSNAME = 'NgxEditor__MenuItem--Icon';
const MENU_ITEM_ICON_CONTAINER_CLASSNAME = 'NgxEditor__MenuItem--IconContainer';
const ACTIVE_MENU_ITEM_CLASSNAME = `${MENU_ITEM_CLASSNAME}--Active`;
const DISABLED_CLASSNAME = 'NgxEditor--Disabled';
class SimpleMenuItem {
    constructor(menuItem, editorView, spec) {
        this.menuItem = menuItem;
        this.editorView = editorView;
        this.spec = spec;
    }
    render() {
        const { dom, update: updateDom } = new MenuItem(this.spec);
        const { schema } = this.editorView.state;
        const { command } = this.setupCommandListeners(dom);
        const update = (state) => {
            const menuItem = this.menuItem;
            let isActive = false;
            const canExecute = command(this.editorView.state, null);
            if (menuItem.type === 'mark') {
                const type = schema.marks[menuItem.key];
                isActive = isMarkActive(state, type);
            }
            if (menuItem.type === 'node') {
                const type = schema.nodes[menuItem.key];
                isActive = isNodeActive(state, type, menuItem.attrs);
            }
            updateDom({
                active: isActive,
                disabled: !canExecute
            });
        };
        return {
            dom,
            update
        };
    }
    setupCommandListeners(dom) {
        const { schema } = this.editorView.state;
        let command;
        if (this.menuItem.type === 'mark') {
            command = toggleMark(schema.marks[this.menuItem.key]);
        }
        if (this.menuItem.type === 'node') {
            const type = schema.nodes[this.menuItem.key];
            if (isListItem(type, schema)) {
                command = toggleList(type, schema.nodes.list_item);
            }
            if (type === schema.nodes.heading) {
                command = toggleBlockType(type, schema.nodes.paragraph, { level: this.menuItem.attrs.level });
            }
            if (type === schema.nodes.blockquote) {
                command = toggleWrap(type);
            }
        }
        dom.addEventListener('mousedown', (e) => {
            e.preventDefault();
            // don't execute if not left click
            if (e.buttons !== 1) {
                return;
            }
            // execute command
            command(this.editorView.state, this.editorView.dispatch);
        });
        return { command };
    }
}

const menuItemsMeta = {
    bold: {
        key: 'strong',
        i18nKey: 'bold',
        icon: 'bold',
        type: 'mark',
    },
    italic: {
        key: 'em',
        i18nKey: 'italics',
        icon: 'italic',
        type: 'mark',
    },
    code: {
        key: 'code',
        i18nKey: 'code',
        icon: 'code',
        type: 'mark',
    },
    ordered_list: {
        key: 'ordered_list',
        i18nKey: 'ordered_list',
        icon: 'ordered_list',
        type: 'node',
    },
    bullet_list: {
        key: 'bullet_list',
        i18nKey: 'bullet_list',
        icon: 'bullet_list',
        type: 'node',
    },
    h1: {
        key: 'heading',
        i18nKey: 'heading',
        attrs: {
            level: 1,
        },
        type: 'node'
    },
    h2: {
        key: 'heading',
        i18nKey: 'heading',
        attrs: {
            level: 2,
        },
        type: 'node'
    },
    h3: {
        key: 'heading',
        i18nKey: 'heading',
        attrs: {
            level: 3,
        },
        type: 'node'
    },
    h4: {
        key: 'heading',
        i18nKey: 'heading',
        attrs: {
            level: 4,
        },
        type: 'node'
    },
    h5: {
        key: 'heading',
        i18nKey: 'heading',
        attrs: {
            level: 5,
        },
        type: 'node'
    },
    h6: {
        key: 'heading',
        i18nKey: 'heading',
        attrs: {
            level: 6,
        },
        type: 'node'
    },
    blockquote: {
        key: 'blockquote',
        i18nKey: 'blockquote',
        icon: 'quote',
        type: 'node'
    },
    link: {
        key: 'link',
        i18nKey: 'link',
        icon: 'link',
        type: 'mark',
        toggleIcon: 'unlink'
    }
};

const DROPDWON_ITEM_CLASSNAME = 'NgxEditor__Dropdown';
const DROPWDOWN_OPEN_CLASSNAME = `${DROPDWON_ITEM_CLASSNAME}--Open`;
const ACTIVE_DROPDOWN_ITEM_CLASSNAME = `${DROPDWON_ITEM_CLASSNAME}--Active`;
const SELECTED_DROPDOWN_ITEM_CLASSNAME = `${DROPDWON_ITEM_CLASSNAME}--Selected`;
class DropDownView {
    constructor(dropdownGroup, dropdownFields, editorView, options) {
        this.updates = [];
        this.dropdownGroup = dropdownGroup;
        this.dropdownFields = dropdownFields;
        this.editorView = editorView;
        this.options = options;
    }
    getWrapperDom() {
        let isDropdownOpen = false;
        const dropdown = document.createElement('div');
        const labels = this.options.labels;
        dropdown.classList.add(DROPDWON_ITEM_CLASSNAME);
        const dropdownText = document.createElement('div');
        dropdownText.classList.add(`${DROPDWON_ITEM_CLASSNAME}__Text`);
        dropdownText.textContent = labels[this.dropdownGroup];
        dropdown.appendChild(dropdownText);
        // create dropdown list
        const dropdownMenu = document.createElement('div');
        dropdownMenu.classList.add(`${DROPDWON_ITEM_CLASSNAME}__DropdownMenu`);
        const mouseDownHandler = (e) => {
            e.preventDefault();
            if (!dropdown.contains(e.target)) {
                closeDropdown();
            }
        };
        const openDropdown = (e) => {
            const target = e.target;
            if (dropdownMenu.contains(target)) {
                return;
            }
            dropdown.classList.add(DROPWDOWN_OPEN_CLASSNAME);
            isDropdownOpen = true;
            window.addEventListener('mousedown', mouseDownHandler);
        };
        const closeDropdown = () => {
            dropdown.classList.remove(DROPWDOWN_OPEN_CLASSNAME);
            isDropdownOpen = false;
            window.removeEventListener('mousedown', mouseDownHandler);
        };
        dropdown.addEventListener('click', (e) => {
            e.preventDefault();
            if (!isDropdownOpen) {
                openDropdown(e);
            }
            else {
                closeDropdown();
            }
        });
        this.dropdownFields.forEach(dropdownItem => {
            const menuItem = menuItemsMeta[dropdownItem];
            let text = labels[menuItem.key];
            if (menuItem.key === 'heading') {
                text += ` ${menuItem.attrs.level}`;
            }
            const spec = {
                classNames: [
                    `${DROPDWON_ITEM_CLASSNAME}__Item`
                ],
                textContent: text,
                attrs: {
                    title: text
                },
                activeClass: ACTIVE_DROPDOWN_ITEM_CLASSNAME,
                disabledClass: DISABLED_CLASSNAME
            };
            const menuItemView = new SimpleMenuItem(menuItem, this.editorView, spec);
            const { update, dom } = menuItemView.render();
            // remove open class once clicked on dropdown value
            dom.addEventListener('click', (e) => {
                e.preventDefault();
                closeDropdown();
            });
            // wrapper to execute when update is called
            const dropUpdate = (state) => {
                update(state);
                // update the dropdown content heading when a class is selected
                const activeEl = dropdownMenu.getElementsByClassName(ACTIVE_DROPDOWN_ITEM_CLASSNAME);
                if (activeEl.length) {
                    const el = activeEl[0];
                    dropdownText.textContent = el.textContent;
                    dropdown.classList.add(SELECTED_DROPDOWN_ITEM_CLASSNAME);
                }
                else {
                    // restore default value
                    dropdownText.textContent = labels[this.dropdownGroup];
                    dropdown.classList.remove(SELECTED_DROPDOWN_ITEM_CLASSNAME);
                }
            };
            dropdownMenu.appendChild(dom);
            this.updates.push(dropUpdate);
        });
        dropdown.appendChild(dropdownMenu);
        return dropdown;
    }
    render() {
        this.dom = this.getWrapperDom();
        return {
            dom: this.dom,
            updates: this.updates
        };
    }
}

const SEPERATOR_CLASSNAME = 'NgxEditor__Seperator';
const getSeperator = () => {
    const div = document.createElement('div');
    div.className = SEPERATOR_CLASSNAME;
    return div;
};
const ɵ0$1 = getSeperator;

const POPOUP_ITEM_CLASSNAME = 'NgxEditor__Popup';
const POPOUP_ITEM_OPEN_CLASSNAME = `${POPOUP_ITEM_CLASSNAME}--Open`;
class Popup {
    constructor() {
        this.show = () => {
            this.dom.classList.toggle(POPOUP_ITEM_OPEN_CLASSNAME, true);
        };
        this.hide = () => {
            this.dom.classList.toggle(POPOUP_ITEM_OPEN_CLASSNAME, false);
        };
        this.dom = document.createElement('div');
        this.dom.classList.add(POPOUP_ITEM_CLASSNAME);
    }
}

const FORM_CLASSNAME = 'NgxEditor__Popup--Form';
const FORMGROUP_CLASSNAME = 'NgxEditor__Popup--FormGroup';
const FORMGROUP_COL_CLASSNAME = 'NgxEditor__Popup--Col';
const FORMGROUP_LABEL_CLASSNAME = 'NgxEditor__Popup--Label';
const createInputElement = (formItem) => {
    var _a, _b, _c, _d;
    const inputwrapper = document.createElement('div');
    inputwrapper.className = FORMGROUP_COL_CLASSNAME;
    const input = document.createElement('input');
    input.type = (_a = formItem.type) !== null && _a !== void 0 ? _a : 'text';
    input.required = (_b = formItem.required) !== null && _b !== void 0 ? _b : true;
    input.placeholder = (_c = formItem.placeholder) !== null && _c !== void 0 ? _c : '';
    input.defaultValue = (_d = formItem.defaultValue) !== null && _d !== void 0 ? _d : '';
    input.name = formItem.name;
    input.autocomplete = 'off';
    const label = document.createElement('label');
    label.textContent = formItem.label;
    label.className = FORMGROUP_LABEL_CLASSNAME;
    inputwrapper.appendChild(label);
    inputwrapper.appendChild(input);
    return inputwrapper;
};
const ɵ0$2 = createInputElement;
const createCheckboxElement = (formItem) => {
    var _a, _b;
    const label = document.createElement('label');
    label.textContent = formItem.label;
    label.style.display = 'flex';
    const input = document.createElement('input');
    input.type = 'checkbox';
    input.required = (_a = formItem.required) !== null && _a !== void 0 ? _a : true;
    input.name = formItem.name;
    input.defaultChecked = (_b = formItem.defaultChecked) !== null && _b !== void 0 ? _b : false;
    label.prepend(input);
    return label;
};
const ɵ1 = createCheckboxElement;
const serializeForm = (formEl) => {
    const data = {};
    for (const el of Array.from(formEl.elements)) {
        if (el.tagName === 'INPUT') {
            const inputEl = el;
            if (inputEl.type === 'checkbox') {
                data[inputEl.name] = inputEl.checked;
            }
            else {
                data[inputEl.name] = inputEl.value;
            }
        }
    }
    return data;
};
const ɵ2 = serializeForm;
class FormView {
    constructor(options) {
        this.render = (formInputs) => {
            this.dom.innerHTML = '';
            formInputs.forEach(formGroup => {
                const groupEl = document.createElement('div');
                groupEl.classList.add(FORMGROUP_CLASSNAME);
                formGroup.forEach(formItem => {
                    let formItemEl;
                    switch (formItem.type) {
                        case 'text':
                        case 'url':
                            formItemEl = createInputElement(formItem);
                            break;
                        case 'checkbox':
                            formItemEl = createCheckboxElement(formItem);
                            break;
                        default:
                            return;
                    }
                    groupEl.appendChild(formItemEl);
                });
                this.dom.appendChild(groupEl);
            });
            const submitButton = document.createElement('button');
            submitButton.type = 'submit';
            submitButton.textContent = 'Submit';
            this.dom.appendChild(submitButton);
        };
        this.dom = document.createElement('form');
        this.dom.classList.add(FORM_CLASSNAME);
        this.dom.onsubmit = (e) => {
            const target = e.target;
            e.preventDefault();
            const data = serializeForm(target);
            options.onSubmit(data);
            target.reset();
        };
        this.render(options.inputs);
    }
}

const getFormInputs = (defaultValue = '') => [
    [
        {
            type: 'url',
            required: true,
            label: 'URL',
            name: 'href'
        }
    ],
    [
        {
            type: 'text',
            required: true,
            label: 'Text',
            name: 'text',
            defaultValue
        }
    ],
    [
        {
            type: 'checkbox',
            required: false,
            defaultChecked: true,
            label: 'Open in new tab',
            name: 'openInNewTab'
        }
    ]
];
const ɵ0$3 = getFormInputs;
const updateLink = (view, data) => {
    const { dispatch, state: { schema, selection, tr } } = view;
    const attrs = {
        title: data.text,
        href: data.href,
        target: data.openInNewTab ? '_blank' : '_self'
    };
    if (selection.empty) {
        const node = schema.text(data.text, [schema.marks.link.create(attrs)]);
        dispatch(tr.replaceSelectionWith(node, false));
    }
    else {
        const command = toggleMark(schema.marks.link, attrs);
        command(view.state, dispatch);
    }
};
const ɵ1$1 = updateLink;
const link$1 = (view, spec) => {
    const { dom, update: updateDom, toggleIcon } = new MenuItem(spec);
    const onSubmit = (data) => {
        updateLink(view, data);
        closePopup();
    };
    const { dom: popupDom, show, hide } = new Popup();
    const { dom: formDom, render: renderForm } = new FormView({ inputs: getFormInputs(), onSubmit });
    let isPopupOpen = false;
    popupDom.appendChild(formDom);
    dom.appendChild(popupDom);
    const mouseDownHandler = (e) => {
        if (!dom.contains(e.target)) {
            closePopup(e);
        }
    };
    const setActiveState = () => {
        updateDom({
            active: isPopupOpen,
            disabled: false
        });
    };
    const showPopup = (e) => {
        e.preventDefault();
        const { state } = view;
        const { doc, selection: { from, to, empty }, schema } = state;
        const isActive = isMarkActive(state, schema.marks.link);
        if (isActive) {
            return removeLink(view);
        }
        const selectedText = !empty ? doc.textBetween(from, to) : '';
        renderForm(getFormInputs(selectedText));
        show();
        isPopupOpen = true;
        setActiveState();
        window.addEventListener('mousedown', mouseDownHandler);
    };
    const closePopup = (e) => {
        if (e && popupDom.contains(e.target)) {
            return;
        }
        e === null || e === void 0 ? void 0 : e.preventDefault();
        hide();
        isPopupOpen = false;
        setActiveState();
        window.removeEventListener('mousedown', mouseDownHandler);
    };
    dom.addEventListener('mousedown', (e) => {
        if (isPopupOpen) {
            closePopup(e);
        }
        else {
            showPopup(e);
        }
    });
    const update = (editorState) => {
        const { schema } = editorState;
        const command = toggleMark(schema.marks.link);
        const canExecute = command(editorState, null);
        const isActive = isMarkActive(editorState, schema.marks.link);
        toggleIcon(isActive);
        updateDom({
            active: isPopupOpen || isActive,
            disabled: !canExecute
        });
    };
    return {
        dom,
        update
    };
};
const ɵ2$1 = link$1;

const DROPDOWN_ITEMS = new Map();
DROPDOWN_ITEMS.set('heading', ['h1', 'h2', 'h3', 'h4', 'h5', 'h6']);
const builtInMenuItems = {
    link: link$1
};
const renderMenu = (options, editorView, menuDom) => {
    const updates = [];
    const toolbar = options.toolbar;
    toolbar.forEach((group, toolbarIndex) => {
        const isLastMenuGroup = toolbar.length - 1 === toolbarIndex;
        group.forEach((toolbarItem, menuIndex) => {
            const isLastMenuItem = group.length - 1 === menuIndex;
            // render dropdown
            if (typeof toolbarItem === 'object') {
                Object.keys(toolbarItem).forEach((dropdownGroup) => {
                    if (DROPDOWN_ITEMS.has(dropdownGroup)) {
                        const dropdown = toolbarItem[dropdownGroup];
                        const dropdownView = new DropDownView(dropdownGroup, dropdown, editorView, options);
                        const rendered = dropdownView.render();
                        updates.push(rendered.updates);
                        menuDom.appendChild(rendered.dom);
                    }
                    else {
                        console.warn('Unkown dropdown group:', dropdownGroup);
                    }
                });
            }
            // render Icons
            if (typeof toolbarItem === 'string') {
                const menuItem = menuItemsMeta[toolbarItem];
                const labels = options.labels;
                if (menuItem) {
                    const spec = {
                        classNames: [
                            MENU_ITEM_CLASSNAME,
                            MENU_ITEM_ICON_CLASSNAME,
                        ],
                        icon: {
                            default: getIconSvg(menuItem.icon),
                            alt: getIconSvg(menuItem.toggleIcon)
                        },
                        attrs: {
                            title: labels[menuItem.i18nKey]
                        },
                        activeClass: ACTIVE_MENU_ITEM_CLASSNAME,
                        disabledClass: DISABLED_CLASSNAME
                    };
                    if (Object.keys(builtInMenuItems).includes(toolbarItem)) {
                        const { dom, update } = builtInMenuItems === null || builtInMenuItems === void 0 ? void 0 : builtInMenuItems[toolbarItem](editorView, spec);
                        menuDom.appendChild(dom);
                        updates.push(update);
                    }
                    else {
                        const menuItemView = new SimpleMenuItem(menuItem, editorView, spec);
                        const { dom, update } = menuItemView.render();
                        menuDom.appendChild(dom);
                        updates.push(update);
                    }
                }
            }
            if (typeof toolbarItem === 'function') {
                const { dom, update } = toolbarItem(editorView, null);
                menuDom.appendChild(dom);
                updates.push(update);
            }
            if (isLastMenuItem && !isLastMenuGroup) {
                const seperatorDom = getSeperator();
                menuDom.appendChild(seperatorDom);
            }
        });
    });
    const combinedUpdates = flatDeep(updates, Infinity);
    return {
        update(state) {
            combinedUpdates.forEach((update) => {
                update(state);
            });
        }
    };
};

class MenuBarView {
    constructor(editorView, options) {
        this.view = editorView;
        this.options = options;
        this.render();
        this.update();
    }
    render() {
        var _a;
        const menuDom = document.createElement('div');
        menuDom.className = 'NgxEditor__MenuBar';
        const { update } = renderMenu(this.options, this.view, menuDom);
        this.updateMenuItems = update;
        (_a = this.view.dom.parentNode) === null || _a === void 0 ? void 0 : _a.insertBefore(menuDom, this.view.dom);
    }
    update() {
        var _a;
        (_a = this.updateMenuItems) === null || _a === void 0 ? void 0 : _a.call(this, this.view.state);
    }
}

const DEFAULT_TOOLBAR = [
    ['bold', 'italic'],
    ['code', 'blockquote'],
    ['ordered_list', 'bullet_list'],
    [{ heading: ['h1', 'h2', 'h3', 'h4', 'h5', 'h6'] }],
    ['link']
];
const DEFAULT_LABELS = {
    bold: 'Bold',
    italics: 'Italics',
    code: 'Code',
    ordered_list: 'Ordered List',
    bullet_list: 'Bullet List',
    heading: 'Heading',
    blockquote: 'Quote',
    link: 'Link'
};
const DEFAULT_OPTIONS = {
    toolbar: DEFAULT_TOOLBAR,
    labels: DEFAULT_LABELS
};
function menuPlugin(options) {
    return new Plugin({
        key: new PluginKey('menu'),
        view(editorView) {
            return new MenuBarView(editorView, options);
        },
    });
}
const menu = (options = DEFAULT_OPTIONS) => {
    const mergedOptions = Object.assign({}, DEFAULT_OPTIONS, options);
    return menuPlugin(mergedOptions);
};
const ɵ0$4 = menu;

class FloatingOptionsView {
    constructor(view) {
        this.bubbleEL = document.createElement('div');
        this.render(view);
        this.update(view);
    }
    render(view) {
        var _a;
        this.bubbleEL.className = 'NgxEditor__FloatingBubble';
        (_a = view.dom.parentNode) === null || _a === void 0 ? void 0 : _a.appendChild(this.bubbleEL);
    }
    setDomPosition(view) {
        // Otherwise, reposition it and update its content
        this.bubbleEL.style.display = '';
        const { bottom, left } = calculateBubblePos(view, this.bubbleEL);
        this.bubbleEL.style.left = left + 'px';
        this.bubbleEL.style.bottom = bottom + 'px';
    }
    createLinkNode(item, removeCB) {
        const el = document.createDocumentFragment();
        const link = document.createElement('a');
        link.href = item.attrs.href;
        link.target = item.attrs.target;
        link.innerText = item.attrs.href;
        link.title = item.attrs.href;
        const commands = document.createElement('div');
        commands.classList.add('commands');
        const removeOpt = document.createElement('button');
        removeOpt.type = 'button';
        removeOpt.classList.add('command');
        removeOpt.textContent = 'Remove';
        removeOpt.addEventListener('mousedown', removeCB, { once: true });
        commands.appendChild(removeOpt);
        el.appendChild(link);
        el.appendChild(commands);
        return el;
    }
    clearBubbleContent() {
        this.bubbleEL.textContent = '';
    }
    hideBubble() {
        this.bubbleEL.style.display = 'none';
    }
    update(view) {
        const { state } = view;
        const { schema } = state;
        if (!schema.marks.link) {
            return;
        }
        const hasFocus = view.hasFocus();
        const isActive = isLinkActive(state);
        const linkMarks = getSelectionMarks(state).filter(mark => mark.type === schema.marks.link);
        // hide for selection and show only for clicks
        if (!hasFocus || !isActive) {
            this.hideBubble();
            return;
        }
        const [linkItem] = linkMarks;
        this.clearBubbleContent();
        const removeCB = (e) => {
            e.preventDefault();
            e.stopPropagation();
            removeLink(view);
            view.focus();
        };
        const el = this.createLinkNode(linkItem, removeCB);
        this.bubbleEL.appendChild(el);
        // update dom position
        this.setDomPosition(view);
    }
    destroy() {
        this.bubbleEL.remove();
    }
}
function linkPlugin() {
    return new Plugin({
        key: new PluginKey('link'),
        view(editorView) {
            return new FloatingOptionsView(editorView);
        },
        props: {
            handleDOMEvents: {
                blur: (view) => {
                    // dummy transaction to update
                    view.dispatch(view.state.tr.setMeta('LINK_PLUGIN_EDITOR_BLUR', true));
                    return false;
                }
            }
        }
    });
}

/**
 * Generated bundle index. Do not edit.
 */

export { MenuItem, linkPlugin as link, menu, placeholderPlugin as placeholder };
//# sourceMappingURL=ngx-editor-plugins.js.map
